#!/usr/bin/perl
# 
# qt.pm - qt module for pbedic
# Miroslav Yordanov 2003
# 
# http://mironcho.sourceforge.net
# mironcholinux@mail.bg
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;
use warnings;

package qt_iface;
use Qt;
use Qt::isa "Qt::Widget";
use Qt::slots
	NewSearch => ['const QString&'],
	GetSelected => ['QListBoxItem *'],
	GetNewFont => [],
	SetTabPos => ['int'],
	PlaySound => [],
	ChangeWordListNum => ['int'],
	ChangeHistoryListNum => ['int'],
	TabChanged => ['QWidget *'];
use Qt::signals
	ChangeText => ['const QString&'],
	ClearListBox => [],
	SetNewWord => ['const QString&'],
	PlaySoundOn => [],
	ExitPbedic => [];
#use Qt::attributes qw(cp1251);

use vars qw($list $font $t $cp1251 $hist $found_sound $img $smile1 $smile2);

# config vars
use vars qw($lo $es $sep $history_num);

sub NEW {
	shift->SUPER::NEW(@_[0..2]);

	$history_num = 50; # default 
	
	$cp1251 = Qt::TextCodec::codecForName("CP1251");
	unless (defined $cp1251) {
		$cp1251 = Qt::TextCodec::codecForName("ISO 8859-1");
	}
	
	$font = Qt::Font("Helvetica", 12, &Qt::Font::Normal);
	
	my $v1 = Qt::VBoxLayout(this);
	my $h1 = Qt::HBoxLayout($v1);
	
	$t = Qt::TabWidget(this);
	$t->setTabPosition($t->Bottom);
	$v1->setMargin(1);
	$h1->addWidget($t);
	
	$main::app->setFont($font);
	$t->setFont($font);

	
	# 
	# First tab contents
	# Dictionary
	#
	
	my $ts1 = Qt::Widget($t);
	
	# GridLayout for first tab
	my $g = Qt::GridLayout($ts1);
	$g->setMargin(3);
	$g->setSpacing(5);

	
	# Top widget and grid
	my $subwt = Qt::Widget($ts1);
	my $gsub1 = Qt::GridLayout($subwt);
	
	# Search box
	my $inp = Qt::LineEdit($subwt);
	$inp->setMaximumWidth(190);
	$inp->setAlignment($inp->AlignLeft);
	$inp->setFocus;
	$gsub1->addMultiCellWidget($inp,0,0,0,1);

	this->connect($inp, SIGNAL('textChanged(const QString&)'), SLOT('NewSearch(const QString&)') );
	$inp->connect(this, SIGNAL('SetNewWord(const QString&)'), SLOT('setText(const QString&)') );

	
	# History combobox
	$hist = Qt::ComboBox(0, $subwt);
	$hist->setMinimumWidth(130);
	$hist->setMaxCount(10);
	$hist->setInsertionPolicy($hist->AtBottom);
	$gsub1->addWidget($hist, 0, 1, $hist->AlignRight);

	$inp->connect($hist, SIGNAL('activated(const QString &)'), SLOT('setText(const QString &)') );
	
	# images
	$smile1 = Qt::Pixmap($main::SHARE_PATH . '/images/wink.gif');
	$smile2 = Qt::Pixmap($main::SHARE_PATH . '/images/biggrin.gif');
	
	# images viewing
	$img = $subwt->Qt::Label;
	$img->setAlignment($img->AlignRight);
	$img->setPixmap($smile1);		# default to wink.gif
	$gsub1->addWidget($img, 0, 2);

	$g->addWidget($subwt,0,0);
	
	# Down widget and grid
	my $subwb = Qt::Widget($ts1);
	my $gsub2 = Qt::GridLayout($subwb);
	$gsub2->setSpacing(5);

	
	# ListBox for next words
	$list = Qt::ListBox($subwb);
	$list->setMinimumWidth(150);
	$list->setMaximumWidth(150);
	$gsub2->addWidget($list,0,0);

	$list->connect(this, SIGNAL('ClearListBox()'),SLOT('clear()') );
	this->connect($list, SIGNAL('clicked(QListBoxItem *)'), SLOT('GetSelected(QListBoxItem *)') );
	
	# Translation viewer ;)
	my $tview = Qt::TextEdit(' ',undef,$subwb);
	$tview->setReadOnly(1);
	#$tview->setAlignment($tview->AlignRight);
	$gsub2->addWidget($tview,0,1);

	$tview->connect(this, SIGNAL('ChangeText(const QString&)'), SLOT('setText(const QString&)') );

	$g->addWidget($subwb,1,0);
	$t->insertTab($ts1, 'Dictionary', 1);

	
	#
	# Second tab contents
	# Configuration
	#
	
	my $ts2 = Qt::Widget($t);
	my $g1 = Qt::GridLayout($ts2,1,2);
	
	my $bgrp1 = Qt::VButtonGroup('Options', $ts2);
	
	# Latin output checkbox
	$lo = Qt::CheckBox('Output Bulgarian as Latin', $bgrp1);

	# "To play, or not to play" sounds ;)
	$es = Qt::CheckBox('Enable Sounds', $bgrp1);
	$es->setChecked(0);	# default to NO sounds

	# Separation ....
	$sep = Qt::CheckBox('Separate different meanings', $bgrp1);
	$sep->setChecked(1);
	
	Qt::Label(' ', $bgrp1);

	my $wordsnum = Qt::Widget($bgrp1);
	my $wordsnumGrid = Qt::GridLayout($wordsnum);
	
	$wordsnumGrid->addWidget(Qt::Label('Words in list box:', $wordsnum), 0, 0);
	
	my $wnum = Qt::SpinBox(10, 500, 10, $wordsnum);
	$wnum->setMaximumWidth(50);
	$wnum->setValue($main::word_list_num);
	this->connect($wnum, SIGNAL('valueChanged(int)'), SLOT('ChangeWordListNum(int)') );
	$wordsnumGrid->addWidget($wnum, 0, 1);
	
	
	my $histnum = Qt::Widget($bgrp1);
	my $histnumGrid = Qt::GridLayout($histnum);
	
	$histnumGrid->addWidget(Qt::Label('Words in history box:', $histnum), 0, 0);
	
	my $hnum = Qt::SpinBox(5, 100, 5, $histnum);
	$hnum->setMaximumWidth(50);
	$hnum->setValue($history_num);
	this->connect($hnum, SIGNAL('valueChanged(int)'), SLOT('ChangeHistoryListNum(int)') );
	$histnumGrid->addWidget($hnum, 0, 1);

	
	my $tabpos = Qt::Widget($bgrp1);
	my $tabposGrid = Qt::GridLayout($tabpos);
	
	$tabposGrid->addWidget(Qt::Label('Tab position:', $tabpos), 0, 0);
	
	my $tpos = Qt::ComboBox(0, $tabpos);
	$tpos->setMaximumWidth(100);
	$tpos->insertItem('Bottom', 0);
	$tpos->insertItem('Top', 1);
	this->connect($tpos, SIGNAL('activated(int)'), SLOT('SetTabPos(int)') );
	$tabposGrid->addWidget($tpos, 0, 1);
	
	Qt::Label(' ',  $bgrp1);

	# Font 
	my $ch_font = Qt::PushButton( 'Change current font', $bgrp1);
	this->connect($ch_font, SIGNAL('clicked()'), SLOT('GetNewFont()') );

	$g1->addWidget($bgrp1, 0, 0);

	my $conf_icon = $ts2->Qt::Label;
	$conf_icon->setPixmap(Qt::Pixmap($main::SHARE_PATH . '/images/config.png'));
	$conf_icon->setAlignment($conf_icon->AlignCenter);
	$g1->addWidget($conf_icon, 0, 1);

	$t->insertTab($ts2, 'Configuration', 2);

	
	my $ts3 = Qt::Widget($t);
	my $g2 = Qt::GridLayout($ts3);

	my $grpbox = Qt::VButtonGroup('About', $ts3);
	my $about_text = $grpbox->Qt::Label;
	#$about_text->setTextFormat(&Qt::RichText);
	my $atext = "\npbedic Qt GUI\n\nPerl Bulgarian English Dictionary\nversion 0.2\n\n\n";
	$atext .= "Author:\nMiroslav Yordanov";
	$about_text->setText($atext);
	$about_text->setAlignment($about_text->AlignCenter);

	my $about_text1 = $grpbox->Qt::Label;
	#$about_text1->setTextFormat(&Qt::RichText);
	$about_text1->setText("<a href=\"http://mironcho.sourceforge.net\">http://mironcho.sourceforge.net</a>");
	$about_text1->setAlignment($about_text1->AlignCenter);

	my $about_text2 = $grpbox->Qt::Label;
	$about_text2->setText("<a href=\"mailto:mironcholinux\@mail.bg\">mironcholinux\@mail.bg</a>");
	$about_text2->setAlignment($about_text2->AlignCenter);
	
	$g2->addWidget($grpbox, 0, 0);
	
	my $about_icon = $ts3->Qt::Label;
	$about_icon->setPixmap(Qt::Pixmap($main::SHARE_PATH . '/images/about.png'));
	$about_icon->resize($about_icon->sizeHint);
	$about_icon->setAlignment($about_icon->AlignCenter);
	$g2->addWidget($about_icon, 0, 1);
	
	$t->insertTab($ts3, 'About', 3);

	my $ts4 = $t->Qt::Label(' ');
	$t->insertTab($ts4, 'Quit', 4);
	this->connect($t, SIGNAL('currentChanged(QWidget *)'), SLOT('TabChanged(QWidget *)') );
	
	this->connect(this, SIGNAL('PlaySoundOn()'), SLOT('PlaySound()') );
	$main::app->connect(this, SIGNAL('ExitPbedic()'), SLOT('quit()') );
}

# My slots
sub NewSearch {
	#print STDERR $cp1251->fromUnicode($_[0]) . "\n";
	$main::word = $cp1251->fromUnicode($_[0], length($_[0]));
	my $r = main::my_find;

	$r =  main::cyr2lat($r) if ($lo->isChecked());
	$r =~ s/\n/\n\n/g  if ($sep->isChecked());

	emit ChangeText($cp1251->toUnicode($r));
	emit ClearListBox();

	main::next_words;
	foreach (@main::word_list) {
		$list->insertItem($cp1251->toUnicode($_));
	}
	
	# "Multimedia" events amd history ComboBox events
	if ($main::ex_found) {
		$img->setPixmap($smile2);
		if (length($main::word) > 1) {
			if (($hist->count()) >= $history_num) {
				$hist->removeItem(0);
			}
			$hist->insertItem($cp1251->toUnicode($main::word));
		}
		
		emit PlaySoundOn() if ($es->isChecked());
	} else {
		$img->setPixmap($smile1);
	}
}

sub GetSelected {
	emit SetNewWord($list->currentText);
}

sub GetNewFont {
	my $ok = undef;
	$font = Qt::FontDialog::getFont($ok, $font, this);
	
	$main::app->setFont($font);
	$t->setFont($font);
}

sub SetTabPos {
	if ($_[0]) {
		$t->setTabPosition($t->Top);
	} else {
		$t->setTabPosition($t->Bottom);
	}
}

# What a slot .... :-)
sub PlaySound {
	system "play $main::SHARE_PATH/sounds/found.wav &";
}

# Setting $main::word_list_num
sub ChangeWordListNum {
	$main::word_list_num = $_[0];
}

# Setting $history_num
sub ChangeHistoryListNum {
	$history_num = $_[0];
}

# Looking forward for pushed 'Quit' tab ...
sub TabChanged {
	if ( 3 == $t->indexOf($_[0])) {
		emit ExitPbedic();
	}
}


1;


##########################################################

package main;
# main
use Qt;
use qt_iface;
use vars qw($app);

$app = Qt::Application;

my $h = qt_iface;
$app->setMainWidget($h);
$h->show;
$h->resize(500,300);
exit $app->exec;

