#!/usr/bin/perl
#
# modules/tk.pm - Tk interface module for pbedic 
# Miroslav Yordanov - 2003
# 
# http://mironcho.sourceforge.net
# mironcholinux@mail.bg
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use warnings;
use strict;

use Tk;
use Tk::Balloon;

my ($r, $t, $down, $smiles, $list, $na_word);
my ($latin_output);

my $about = "pbedic v0.2";
my $hl_color = 'DodgerBlue1';
my $hlf_color = 'snow1';
my $font_en = '-*-helvetica-normal-r-12-*-*-*-*-*-*-*-*-*';
my $font = '-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-microsoft-cp1251';
my @mset = (-font => $font_en, '-tearoff', 0, '-underline', 0, '-relief', 'flat');
my @mset1 = (-font => $font_en);

my $m = MainWindow->new(-title => 'pbedic');

$m->setPalette(background => 'grey90',
	activeBackground => $hl_color,
	activeForeground => $hlf_color,
	selectBackground => $hl_color,
	selectForeground => $hlf_color
);

# Menubar
my $menubar = $m->Frame(-relief => 'groove', -borderwidth => 1);
$menubar->grid(-sticky => 'we');
$menubar->gridColumnconfigure(2, -weight => 1);

# File menu
my $file = $menubar->Menubutton(-text => 'File', -menuitems =>
	[
		[Button => '~Quit', -command => [$m, 'destroy'], @mset1 ]
	], @mset
)->grid(-row => 0, -column => 0, -sticky => 'w');

# Settings menu
my $options = $menubar->Menubutton(-text => 'Options', -menuitems =>
	[
		[Button => '~Settings', @mset1, -command => sub{tk_gui_conf();} ]
	], @mset
)->grid(-row => 0, -column => 1, -sticky => 'w');

# Help menu
my $help = $menubar->Menubutton(-text => 'Help', -menuitems =>
	[
		[Button => '~About', -command => sub{about_dialog();}, @mset1]
	], @mset
)->grid(-row => 0, -column => 2, -sticky => 'e');


# Top
my $top = $m->Frame;
$top->grid(-sticky => 'we', -pady => 2);
$top->gridColumnconfigure(2, -weight => 1);

my $e = $top->Entry( -textvariable => \$word,
	-width => 30, -background => 'white',
	-font => $font,
	-relief => 'groove');
$e->grid(-sticky => 'w', -column => 0, -row => 0, -padx => 3);

# Search button
my $bs = $top->Button(-text => 'Search', -command => sub{
		# Main search function 
		$r=my_find();
		return if ($r eq ' ');

		if ($ex_found) {
			$smiles->configure(-image => 'smile2');
		} else {
			$smiles->configure(-image => 'smile1');
		}
		
		next_words();
		$list->delete(0, 'end');
		$list->insert(0, @word_list);
		
		$r =~ s/\n/\n\n/g ;
		$r = cyr2lat($r) if ($latin_output);
		$t->delete('0.0', 'end');
		$t->insert('0.0', "$r\n\n");
	}, -padx => 5, -pady => 1, -relief => 'groove'
);
$bs->grid(-sticky => 'w', -column => 1, -row => 0, -padx => 2);

# Clear button
my $bc = $top->Button(-text => 'Clear', -padx => 5,
	-command => sub{$word = '';},
	-relief => 'groove',
	-pady => 1
	);
$bc->grid(-row => 0, -column => 2, -sticky => 'w', -padx => 2);

$top->Photo('smile1', -file => "$SHARE_PATH/images/wink.gif");
$top->Photo('smile2', -file => "$SHARE_PATH/images/biggrin.gif");
$smiles = $top->Label( -image => 'smile1', -relief => 'flat');
$smiles->grid(-row => 0, -column => 3, -sticky => 'e', -padx => 5);

# Key events
$m->bind('<KeyPress>' => sub{$bs->invoke();} );
$m->bind('<Mod1-F4>' => [$m, 'destroy']);


$down = $m->Frame;
$down->grid(-sticky => 'we');

$list = $down->Scrolled('Listbox', -width => 20,
	-height => 15, -setgrid => 1,
	-scrollbars => 'e', -font => $font,
	-background => 'white',
	-relief => 'groove'
);
$list->grid(-row => 0, -column => 0, -sticky => 'wesn', -padx => 1, -pady => 1);
$list->bind('<Double-1>' =>
	sub {
		$word = $_[0]->get('active');
		$bs->invoke();
	}
);

$t = $down->Scrolled('Text',
	-font => $font,
	-scrollbars => 'e',
	-padx => 5 , -pady => 5, -relief => 'groove',
	-width => 50, -height => 15,
	-background => 'white'
);
$t->grid(-column => 1, -row => 0, -sticky => 'wesn', -padx => 1, -pady => 1);

$e->focus;

# Baloons ( aka tooltips ;)
my @blnset = (
	'-initwait', 1500, # waiting time
	'-balloonposition', 'mouse',
	'-state', 'balloon'
);
my $bln = $m->Balloon;
my $bln2 = $m->Balloon;

$bln->attach($bs, @blnset,
	-msg => "Press this button\n to search for entered word"
);
$bln->attach($bc, @blnset,
	-msg => 'Clear entered word'
);
$bln->attach($file, @blnset,
	-msg => 'File menu'
);
$bln->attach($options, @blnset,
	-msg => 'Options menu'
);
$bln->attach($help, @blnset,
	-msg => 'Help menu'
);
$bln->attach($e, @blnset,
	-msg => 'Enter word here'
);

my $msg = '';
my $yy = '';
my $list_pos ;

balloon_attach();

$bln->attach($list,
	-initwait => 0,
	-balloonposition => 'mouse',
	-state => 'balloon',
	#-msg => \$msg,
	-postcommand => sub {
		if ((scalar @word_list) > 0) {
			$list->SelectionClear;
			$yy = $list->pointery - $list->rooty;
			$list_pos = $list->nearest($yy);
			$list->selectionSet($list_pos, $list_pos);
			return 1;
		}
	},
	-motioncommand => sub {
		$bln2->detach($list);
		balloon_attach();
		return 0;
	}
);



$m->OnDestroy(sub {$bln->destroy;});
$m->OnDestroy(sub {$bln2->destroy;});


MainLoop;

sub balloon_attach {
	$bln2->attach($list,
		-initwait => 1500,
		-balloonposition => 'mouse',
		-state => 'balloon',
		-msg => \$msg,
		-postcommand => sub {
			if ((scalar @word_list) > 0) {
				my $yy = $list->pointery - $list->rooty;
				$list_pos = $list->nearest($yy);
				$msg = "Double click to see\n\"$word_list[$list_pos]\" translation";
				return 1;
			} else {
				$msg = "Here will be listed\nnext words";
				return 1;
			}
		}
	);

}

# Function for creating config window
sub tk_gui_conf {
	my $i = 0;
	my $tl = $m->Toplevel(-title => 'pbedic Tk GUI configuration',
			-width => 500, -height => 300) ;
	my $conf = $tl->Frame(
		-width => 500,
		-height => 300,
		-relief => 'groove',
		-borderwidth => 1
	);
	$conf->grid(-sticky => 'wens', -padx => 2, -pady => 1);
	
	my $bl = $conf->Checkbutton(
		-text => 'Output Bulgarian as Latin',
		-font => $font_en,
		-variable => \$latin_output,
		-relief => 'flat',
		-command => sub{$bs->invoke;}
	);
	$bl->grid(-row => $i, -column => 0, -sticky => 'w', -padx => 15, -pady => 1);
	$i++ ;

	my $space = $conf->Label(
		-text => '         ',
		-relief => 'flat'
	);
	$space->grid(-row => $i, -column => 0, -sticky => 'ew', -pady => 0);
	$i++ ;

	my $bc = $conf->Button(
		-text => 'Close',
		-padx => 5,
		-pady => 1,
		-relief => 'groove',
		-command => [$tl, 'destroy']
	);
	$bc->grid(-row => $i, -column => 0, -padx => 10, -pady => 2);
}


# function for creating About window
sub about_dialog {
	my $tl = $m->Toplevel(-title => 'About pbedic');
	my $a = $tl->Frame(
		-relief => 'groove',
		-borderwidth => 1
	);
	$a->grid(-sticky => 'wesn');

	my $text = "$about by mironcho\n\nPerl Bulgarian English Dictionary\n\n";
	$text .= "author: Miroslav Yordanov\nmironcholinux\@mail.bg\n";
	$text .= "http://mironcho.sourceforge.net";
	$a->Label(
		-text => $text,
		-relief => 'flat',
		-font => $font_en
	)->grid(-row => 0, -column => 0, -sticky => 'we', -padx => 10, -pady => 10);

	my $b = $a->Button(
		-text => 'Close',
		-relief => 'groove',
		-padx => 2,
		-pady => 1,
		-command => [$tl, 'destroy']
	);
	$b->grid(-row => 1, -column => 0, -padx => 30, -pady => 2);
	#$b->focus;
}

1;
